@@ -639,11 +639,11 @@ tree_event (Gpm_Event * event, void *data)
     if (!mouse_global_in_widget (event, w))
         return MOU_UNHANDLED;
 
-    local = mouse_get_local (event, w);
+    /* rest of the upper frame - call menu */
+    if (tree->is_panel && (event->type & GPM_DOWN) != 0 && event->y == w->owner->y + 1)
+        return MOU_UNHANDLED;
 
-    /* rest of the upper frame, the menu is invisible - call menu */
-    if (tree->is_panel && (local.type & GPM_DOWN) != 0 && local.y == 1 && !menubar_visible)
-        return the_menubar->widget.mouse (event, the_menubar);
+    local = mouse_get_local (event, w);
 
     if ((local.type & GPM_UP) == 0)
         return MOU_NORMAL;
